@@ -1,3 +1,5 @@
+#![feature(btree_drain_filter)]
+#![feature(map_first_last)]
 #![feature(repr_simd)]
 #![feature(test)]
 
@@ -5,8 +7,8 @@ extern crate test;
 
 mod btree;
 mod linked_list;
-mod string;
-mod str;
 mod slice;
+mod str;
+mod string;
 mod vec;
 mod vec_deque;
